// Methyl spectrum model of FrFe94 according to RoNG05
// S(q,w) = f_q int dE g(E) Lor(w,tau) with tau(E)=tau_0 exp(E/kT)
// g(E) is a Gaussian

// gcc -lm methyl-frick.c

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define K 99
#define Nmax 10000
#define SQR(x) ((x)*(x))

double read_par_d( char* name )
{
    double ret;
    if( scanf( "%lg", &ret ) != 1 ){
        fprintf( stderr, "failed reading %s\n", name );
        exit(-1);
    }
    return ret;
}

int read_par_i( char* name )
{
    int ret;
    if( scanf( "%i", &ret ) != 1 ){
        fprintf( stderr, "failed reading %s\n", name );
        exit(-1);
    }
    return ret;
}

int main()
{
    int M, N, k, i;
    double q, T, Emean, Esigma, tau0, du2dT; // the parameters
    double x[Nmax], y[Nmax];
    double gtot, wgt, E, tau;

    M = read_par_i( "M" );
    if( M!=6 ){
        fprintf( stderr,
                 "Parameters: q, T, Emean, Esigma, tau_0, d<u^2>/dT\n" );
        exit(-1);
    }
    q      = read_par_d( "q" );
    T      = read_par_d( "T" );
    Emean  = read_par_d( "Emean" );
    Esigma = read_par_d( "Esigma" );
    tau0   = read_par_d( "tau_0" );
    du2dT  = read_par_d( "d<u^2>/dT" );

    N = read_par_i( "N" );
    if( N>Nmax ){
        fprintf( stderr, "N exceeds Nmax\n" );
        exit(-1);
    }

    for( i=0; i<N; ++i ){
        if( scanf( "%lg", x+i )!=1 ){
            fprintf( stderr, "failed reading x[%i]\n", i );
            exit(-1);
        }
        y[i] = 0;
    }
    gtot = 0;
    for( k=0; k<K; ++k ){
        E = Emean + (k-K/2)*7.0/K*Esigma;
        wgt = exp( -SQR((E-Emean)/Esigma)/2 );
        gtot += wgt;
        tau = tau0 * exp(-E/T);
        for( i=0; i<N; ++i ){
            y[i] += wgt*tau/(1+SQR(tau*x[i]));
        }
    }
    for( i=0; i<N; ++i ){
        y[i] /= gtot;
        printf( "%g\n", y[i] );
    }
}
