// g++ cofu.cpp -lgsl -lgslcblas -lcerf

#include <math.h>
#include <cerf.h>
#include <stdio.h>
#include <cstdlib>
#include <iostream>
#include <vector>
#include <sstream>
#include <gsl/gsl_integration.h>

using namespace std;

const double PI = M_PI;
#define nwork 20000
#define ndim 1
#define nj 95

typedef struct {
    vector<double>* P;
    gsl_integration_workspace* work[ndim];
} DataType;
    
double f0( double x, double s ) // angle dis
{
    return s/(2*PI)*pow(x+s*s,-1.5);
}

double f1( double x, double s ) // res
{
    return exp(-x*x/2/s/s)/sqrt(2*PI*s*s);
}

double kernel( double t, void* pD )
{
    DataType* D = (DataType*)pD;
    double x  = (*(D->P))[0];
    double sr = (*(D->P))[3];
    double res = 0;
    for( int j=0; j<nj; ++j )
        res += (*(D->P))[4+j] * f1( x+18.*(-1+2.*j/(nj-1)), sr );
    return res;
}

/*
double inner_integration( double to, void* pD )
{
    DataType* D = (DataType*)pD;
    gsl_function F;
    F.function = kernel;
    F.params = pD;
    double val, err;
    (*(D->P))[1] = to;
    gsl_integration_qagiu( &F, 0., 1e-16, 1e-6, nwork, D->work[1], &val, &err );
    return val;
}
*/

double compute_y( double x, DataType& D )
{
    gsl_function F;
    F.function = kernel;
    F.params = (void*)&D;
    double val, err;
    (*(D.P))[0] = x;
    gsl_integration_qagiu( &F, 0., 1e-16, 1e-6, nwork, D.work[0], &val, &err );
    return (*(D.P))[1] + (*(D.P))[2] * val;
}

int main()
{
    int npar;
    cin >> npar;
    if( npar!=nj+3 ) {
        cerr << "Unexpected #pars\n";
        return -1;
    }
    vector<double> P(npar+ndim);
    for( int ip=0; ip<npar; ++ip )
        cin >> P[ndim+ip];
    int nx;
    cin >> nx;
    vector<double> X(nx);
    vector<double> Y(nx);
    for( int i=0; i<nx; ++i )
        cin >> X[i];

    DataType D;
    D.P = &P;
    for( int iw=0; iw<ndim; ++iw )
        D.work[iw] = gsl_integration_workspace_alloc( nwork );
    for( int i=0; i<nx; ++i )
        Y[i] = compute_y( X[i], D );
    for( int iw=0; iw<ndim; ++iw )
        gsl_integration_workspace_free( D.work[iw] );
    
    for( int i=0; i<nx; ++i )
        cout << X[i] << " " << Y[i] << "\n";
}
