//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis                         
//*  (C) Joachim Wuttke 2001-                                               
//*  http://apps.jcns.fz-juelich.de/frida                                   
//**************************************************************************************************

//! \file  fassign.cpp
//! \brief Collection of assignment functions.

#include <cmath>
#include <iostream>
#include <map>
#include <vector>

#include "defs.hpp"
#include "func.hpp"
#include "ptr.hpp"
#include "obj.hpp"
#include "fregistry.hpp"
#include "reg.hpp"

template<class A>
A func_assign (string key, A v)
{
    GReg->register_scalar( key, CObj::objectify(v) );
    return v;
}
void func_assign(double& r, double& dr, string key, double v, double dv)
{
    GReg->register_scalar( key, CObj::objectify(v, dv) );
    r = v; dr = dv;
}

int func_defined (string key)
{
    return GReg->find( key ) ? 1 : 0;
}

void func_undef (string key)
{
    bool ret = GReg->undef( key );
    if( !ret )
        throw "key " + key + " was not defined";
}

void fassign_initialize()
{

#define G GFuncRegistry

    G->register_fct_meta ( "=",   2, "a=b     [assignment]", 14 );
    G->register_fct_i_si ( "=",  func_assign );
    G->register_fct_d_sd ( "=",  func_assign );
    G->register_fct_e_se ( "=",  func_assign );
    G->register_fct_s_ss ( "=",  func_assign );

    G->register_fct_meta ( "defined", 1, "(s): 1 if there is a variable s" );
    G->register_fct_i_s  ( "defined",  func_defined );
    G->register_fct_meta ( "undef", 1, "(s): undefine variable s, return 1 upon success" );
    G->register_fct_0_s  ( "undef",  func_undef );
    
}
