//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  var.hpp
//! \brief Variable reference CVar.


//! Variable type (and number, if applicable).

class CVar {
    // Variables may be unnumbered (x,y,i,j,k,c,...) or numbered (z0,p0,r0)
 public:
    enum TTyp {
        _NOREF=0,
        _X, _Y, _Z, _R, // data
        _DY,            // associated error
        _C, _CP,        // curve, curve parameters
        _COUTCOME, _CCHI2, _CR2, // fit outcome
        _K, _J, _I,     // indices for file, spectrum, point
        _NK, _NJ, _NI   // number of files, of spectra in file, of points in spectrum
    };
    TTyp typ;
    int num;

    CVar( TTyp _typ=_NOREF, int _num=0 ) : typ(_typ), num(_num) {};
    CVar( const string s );

    bool is_errvar() const { return typ==_DY; }
    bool pointwise() const { return typ==_X || typ==_Y || typ==_DY || typ==_I; };
    bool numbered() const { return typ==_CP || typ==_Z; };

    string var_info() const;

 private:
    static int request_num( string s, int pos );
};
